import React, { useState, useContext, forwardRef } from 'react'
import PropTypes from 'prop-types'
import { ThemeContext } from '../utils'

/**
 * Display a standardized action (e.g. a button).
 * 
 * @param  {string} props.element  - Use a different base element
 * @param  {string} props.align    - Pass `right` to push the action to the right of a container
 * @param  {object} props.style    - Override or add any additional styles
 * @param  {object} props.children - Any valid react children
 * @return {object}                - React markup
 */
const Action = forwardRef(({
    element: Element = 'div',
    align = 'left',
    style = {},
    children,
    ...props
}, ref) => {
    const [focused, setFocused] = useState(false)
    const {neutral} = useContext(ThemeContext)

    return (
        <Element
            {...props}
            ref={ref}
            tabIndex={0}
            onFocus={e => setFocused(true)}
            onBlur={e => setFocused(false)}
            style={{
                display: 'inline-flex',
                alignItems: 'center',
                boxSizing: 'content-box',
                height: 16,
                padding: '8.5px 16px 7.5px',
                marginLeft: align === 'right' ? 'auto' : undefined,
                marginRight: 12,
                borderRadius: 20,
                borderWidth: 0,
                boxShadow: focused ? `0 0 10px ${neutral[3]}` : `none`,
                outline: 'none',
                fontFamily: 'inherit',
                fontWeight: 'inherit',
                lineHeight: 1,
                caretColor: neutral[neutral.length - 2],
                background: neutral[0],
                transition: 'all 250ms',
                ...style
            }}
        >
            { children }
        </Element>
    )
})

Action.propTypes = {
    element: PropTypes.string,
    align: PropTypes.oneOf(['left', 'right']),
    style: PropTypes.object,
    children: PropTypes.node
}

export default Action