import React, { useRef } from 'react'
import PropTypes from 'prop-types'
import { useEffect } from 'react'
import { BenchmarkContext } from '../utils/index.js'
import { simplify } from '../utils/children'

/**
 * Benchmark a component's render speed.
 * 
 * @param  {function} props.render    - Typically `ReactDOM.render` but you could use something else
 * @param  {function} props.onRender  - Called every time the component is re-rendered (useBenchmark)
 * @param  {function} props.onAnimate - Called every time a CSS animation or transition is detected
 * @param  {function} props.onMeasure - Called with the render speed and `children`
 * @param  {object}   props.children  - An instance of the component to test
 * @return {object}                   - Whatever is given as `children`
 */
const Benchmark = ({
    render,
    onRender,
    onAnimate,
    onMeasure,
    children = null
}) => {
    const ref = useRef(null)
    const start = performance.now()

    useEffect(() => {
        const tree = (
            <BenchmarkContext.Provider value={{ onRender }}>
                {children}
            </BenchmarkContext.Provider>
        )

        render(tree, ref.current)
        onMeasure(performance.now() - start, children)
    })

    return (
        <div
            ref={ref}
            onTransitionEnd={onAnimate}
            onAnimationEnd={onAnimate}
        />
    )
}

Benchmark.propTypes = {
    render: PropTypes.func.isRequired,
    onRender: PropTypes.func.isRequired,
    onAnimate: PropTypes.func.isRequired,
    onMeasure: PropTypes.func.isRequired,
    children: PropTypes.node
}

export default React.memo(Benchmark, (prev, next) => (
    JSON.stringify(simplify(prev.children)) ===
    JSON.stringify(simplify(next.children))
))