import React, { useState, useRef } from 'react'
import PropTypes from 'prop-types'
import Benchmark from './benchmark.jsx'
import Diagnostic from './diagnostic.jsx'

/**
 * An isolated testing ground for components. Render your component as a child
 * to get diagnostics and optimization tips.
 * 
 * @param  {function} props.render   - Passed through to the `Benchmark` component
 * @param  {object}   props.children - A single react node
 * @return {object}                  - React markup
 */
const Lab = ({
    render,
    children
}) => {
    const container = useRef(null)
    const [speed, setSpeed] = useState(null)
    const [elements, setElements] = useState(null)
    let [animations, setAnimations] = useState(0)
    let [renders, setRenders] = useState(0)

    return (
        <main
            style={{
                display: 'flex',
                minHeight: '100vh',
                alignItems: 'center',
                justifyContent: 'center'
            }}
        >
            <div
                ref={container}
                style={{
                    display: 'flex',
                    flexWrap: 'wrap',
                    maxWidth: 720
                }}
            >
                <Benchmark
                    render={render}
                    onRender={() => setRenders(++renders)}
                    onAnimate={e => setAnimations(++animations)}
                    onMeasure={time => {
                        setTimeout(() => {
                            const nodes = container.current.querySelectorAll('*')
                            const children = Array.from(nodes)
                            const index = children.findIndex(el => el.id === 'benchmark')
                            const measured = children.slice(0, index)

                            setSpeed(Math.floor(time))
                            setElements(measured.length)
                        })
                    }}
                >
                    {children}
                </Benchmark>
                <aside
                    id="benchmark"
                    style={{
                        marginLeft: 32
                    }}
                >
                    <h1
                        style={{
                            marginLeft: -1,
                            fontSize: 30,
                            fontWeight: 700
                        }}
                    >
                        Diagnostics
                    </h1>
                    <Diagnostic
                        label="Render Speed"
                        value={speed}
                        units="ms"
                        warning={50}
                        error={100}
                        max={150}
                    >
                        Render speed is particularly important for components
                        that are shown in multiples. Examples include cards in
                        a grid, rows in a table, or items in a list.
                    </Diagnostic>
                    <Diagnostic
                        label="Number of Elements"
                        value={elements}
                        warning={40}
                        error={75}
                        max={100}
                    >
                        This statistic can help identify overly complex markup
                        structures. You should strive to keep your hierarchy
                        relatively flat, simple and semantic.
                    </Diagnostic>
                    <Diagnostic
                        label="Number of Animations"
                        value={animations}
                        warning={4}
                        error={8}
                        max={10}
                    >
                        Too many CSS animations and transitions can cause
                        layout thrashing, particularly while scrolling. Try to
                        limit most of your animations to the interactions after
                        the initial mount.
                    </Diagnostic>
                    <Diagnostic
                        label="Number of Rerenders"
                        value={renders - 1}
                        warning={1}
                        error={3}
                        max={5}
                        style={!renders ? { display: 'none' } : null}
                    >
                        Exceeded the recommended max rerenders (1). In most
                        cases you shouldn't need to rerender at all within a
                        view-focused component.
                        <br />
                        <a
                            href="..."
                            style={{
                                display: 'inline-block',
                                marginTop: 4,
                                fontWeight: 400,
                                textDecoration: 'none',
                                color: '#3498DB'
                            }}
                        >
                            Click here for more details
                        </a>
                    </Diagnostic>
                </aside>
            </div>
        </main>
    )
}

Lab.propTypes = {
    render: PropTypes.func.isRequired,
    children: PropTypes.node
}

export default Lab